class Stage01 implements GameState {
  Backdrop bg;
  TheEye eye;
  ArrayList drops;
  PImage doll;
  ArrayList people;
  ArrayList souls;
  
  void setup() {
    bg = new Backdrop();
    eye = new TheEye(this);
    drops = new ArrayList();
    doll = loadImage("person.png");
    people = new ArrayList();
    souls = new ArrayList();
    bg.setup();
    eye.setup();
    for (int i = 0; i < 10; i++) {
      people.add(new Person(random(100, width-100), 430., this));
    }
  }
  
  void cleanup() {
    bg = null;
    eye = null;
  }
  
  void draw() {
    // draw and update everything
    // (it's tangled; we should avoid this but oh well)
    bg.draw();
    eye.draw();
    for (int i = people.size()-1; i >= 0; i--) { 
      Person p = (Person) people.get(i);
      p.draw();
      // this is the kind of implementation exposure we should avoid
      if (p.dead) {
        people.remove(i);
      }
    }
    for (int i = drops.size()-1; i >= 0; i--) { 
      Drop drop = (Drop) drops.get(i);
      drop.draw();
      if (drop.dead) {
        drops.remove(i);
      }
    }
    for (int i = souls.size()-1; i >= 0; i--) { 
      Soul s = (Soul) souls.get(i);
      s.draw();
      if (s.dead) {
        souls.remove(i);
      }
    }
    // check collisions between drops and people
    for (int i = drops.size()-1; i >= 0; i--) { 
      Drop drop = (Drop) drops.get(i);
      for (int j = people.size()-1; j >= 0; j--) { 
        Person p = (Person) people.get(j);
        if (dist(drop.x, drop.y, p.x, p.y) < 30) {
          people.remove(j);
          drops.remove(i);
          souls.add(new Soul(p.x, p.y));
          break;
        }
      }
    }
    if (people.size() < 10) {
      people.add(new Person(random(100, width-100), 430., this));
    }
    // check collisions between souls and the eye
    for (int i = souls.size()-1; i >= 0; i--) { 
      Soul s = (Soul) souls.get(i);
      if (dist(s.x, s.y, eye.eyeX, eye.eyeHeight) < 15 + eye.eyeDiameter/2) {
        eye.power = true;
        souls.remove(i);
      }
    }
    // check collisions between people and the eye
    if (eye.eyeX > eye.shadowHeight - eye.eyeDiameter/2) {
      for (int i = people.size()-1; i >= 0; i--) { 
        Person p = (Person) people.get(i);
        if (dist(p.x, p.y, eye.eyeX, eye.eyeHeight) < 20 + eye.eyeDiameter/2) {
          gameOver();
          break;
        }
      }
    }
  }
  
  void keyPressed() {
    eye.keyPressed();
  }
  
  void keyReleased() {
    eye.keyReleased();
  }
  
  void createDrop(float x, float y) {
    Drop d = new Drop(x, y);
    drops.add(d);
  }
}

class Backdrop {
  int[] buildingHeights;
  color[] buildingColors;
  int buildingWidth;
  int buildingOffset;

  void setup() {
    buildingHeights = new int[6];
    buildingColors = new color[6];
    buildingWidth = width/5;
    for (int i = 0; i < 6; i++) {
      buildingHeights[i] = (int)random(120, 200);
      buildingColors[i] = lerpColor(color(#9BD5DB), color(#DBD99B), random(1));
    }
    buildingOffset = 0;
  }
  
  void draw() {
    pushStyle();
    background(#D1E3FA);
    noStroke();
    int bx = buildingOffset;
    int by = height/2;
    for (int i = 0; i < 6; i++) {
      fill(buildingColors[i]);
      rect(bx, by - buildingHeights[i], buildingWidth, buildingHeights[i]);
      bx += buildingWidth;
    }
    fill(#9B9291);
    rect(0, by, width, by);
    buildingOffset-=2;
    if (buildingOffset <= -buildingWidth) {
      buildingOffset += buildingWidth;
      for (int i = 0; i < 5; i++) {
        buildingHeights[i] = buildingHeights[i+1];
        buildingColors[i] = buildingColors[i+1];
      }
      buildingHeights[5] = (int)random(120, 200);
      buildingColors[5] = lerpColor(color(#9BD5DB), color(#DBD99B), random(1));
    }
    popStyle();
  }
}

class TheEye {
  Stage01 parent;
  
  float eyeX, thrust, speed;
  float eyeHeight, minHeight, maxHeight;
  int shadowHeight;
  int eyeDiameter;

  boolean power;
  float steam;
  
  boolean shoot;
  
  TheEye(Stage01 parent) {
    this.parent = parent;
  }
  
  void setup() {
    eyeX = width/2;
    eyeHeight = 200;
    shadowHeight = 450;
    eyeDiameter = 150;
    minHeight = shadowHeight - eyeDiameter/2;
    maxHeight = eyeDiameter/2;
    steam = 0;
    power = false;
    thrust = 0;
    speed = 0;
    shoot = false;
  }
  
  void draw() {
    // horizontal movement
    eyeX = eyeX + speed;
    if (eyeX < eyeDiameter/2) 
      eyeX = eyeDiameter/2;
    else if (eyeX > width - eyeDiameter/2) 
      eyeX = width - eyeDiameter/2;
    speed = speed + thrust - 0.1*speed;
    if (speed < -4) 
      speed = -4;
    else if (speed > 4) 
      speed = 4;
    // vertical movement
    eyeHeight += steam;
    if (eyeHeight < maxHeight)
      eyeHeight = maxHeight;
    else if (eyeHeight > minHeight)
      eyeHeight = minHeight;
    if (power) {
      power = false;
      steam = -1.5;
    }
    else {
      steam = steam + 0.02;
    }
    // check fire
    if (shoot) {
      shoot = false;
      steam += 0.04;
      // this is the kind of dependency we should try to minimize
      parent.createDrop(eyeX, eyeHeight);
    }
    if (steam > 4) 
      steam = 4;
    
    pushStyle();
    // draw the shadow
    fill(59, 52, 51, 128);
    int shadowLength = (int)lerp(eyeDiameter, 1.5*eyeDiameter, (minHeight - eyeHeight)/(minHeight - maxHeight));
    ellipse(eyeX, shadowHeight, shadowLength, shadowLength/3);
    // draw the eye
    fill(255);
    ellipse(eyeX, eyeHeight, eyeDiameter, eyeDiameter);
    fill(#645553);
    ellipse(eyeX, eyeHeight, eyeDiameter/3, eyeDiameter/3);
    fill(0);
    ellipse(eyeX, eyeHeight, 20, 20);
    popStyle();
  }
  
  void keyPressed() {
    if (key == CODED) {
      switch (keyCode) {
        case LEFT:
          thrust = -1;
          break;
        case RIGHT:
          thrust = 1;
          break;
      }
    }
  }
  
  void keyReleased() {
    if (key == CODED) {
      switch (keyCode) {
        case LEFT:
        case RIGHT:
          thrust = 0;
          break;
      }
    }
    else if (key == ' ')
      shoot = true;
  }
}

class Drop {
  float x, y;
  boolean dead;
  
  Drop(float x, float y) {
    this.x = x;
    this.y = y;
    dead = false;
  }
  
  void draw() {
    y += 4;
    if (y >= 450)
      dead = true;
    pushStyle();
    noStroke();
    fill(#F0EA46);
    ellipse(x, y, 20, 20);
    popStyle();
  }
}

class Person {
  float x, y, speed;
  boolean dead;
  Stage01 parent;
  
  Person(float x, float y, Stage01 parent) {
    this.x = x;
    this.y = y;
    speed = (random(100)>50)?random(1, 5):random(-5, -1);
    this.parent = parent;
    dead = false;
  }
  
  void draw() {
    x += speed;
    // this is the kind of "magic number" we should eliminate
    if (x < 20 || x > width-20)
      speed = -speed;
    pushStyle();
    imageMode(CENTER);
    image(parent.doll, x, y);
    popStyle();
  }
}

class Soul {
  float x, y;
  boolean dead;
  
  Soul(float x, float y) {
    this.x = x;
    this.y = y;
    dead = false;
  }
  
  void draw() {
    y -= 2;
    if (y < 0)
      dead = true;
    pushStyle();
    noFill();
    stroke(255);
    strokeWeight(3);
    ellipse(x, y, 30, 30);
    popStyle();
  }
}

